@@ -129,6 +129,9 @@ static const gchar introspection_xml_modemmanager[] =
   "      <arg type='s' name='setting' direction='in'/>"
   "      <arg type='v' name='value' direction='in'/>"
   "    </method>"
+  "    <method name='ChangeAllSettings'>"
+  "      <arg type='a{sv}' name='options' direction='in'/>"
+  "    </method>"
   "    <method name='ProcessMessageQueue'>"
   "      <annotation name='org.ofono.mms.ModemManager' value='OnMethod'/>"
   "    </method>"
@@ -163,6 +166,90 @@ static void mmsd_connect_to_sms_wap(void);
 static void mmsd_disconnect_from_sms_wap(void);
 static void mmsd_get_all_sms(void);
 
+static void
+save_string_settings (GKeyFile		 *key_file,
+		      const gchar	 *group_name,
+		      const gchar	 *key,
+		      const gchar	 *value,
+		      gchar		**location)
+{
+	g_free(*location);
+	*location = g_strdup(value);
+	DBG("%s set to '%s'", key, value);
+
+	if (!value || !*value)
+		value = "NULL";
+	g_key_file_set_string(key_file, group_name, key, value);
+}
+
+static void
+save_boolean_settings (GKeyFile		*key_file,
+		       const gchar	*group_name,
+		       const gchar	*key,
+		       gboolean		 value,
+		       gboolean		*location)
+{
+	*location = value;
+	DBG("%s set to %s", key, value ? "TRUE" : "FALSE");
+	g_key_file_set_boolean(key_file, group_name, key, value);
+}
+
+static gboolean
+dbus_change_settings (const char	*key,
+		      GVariant		*value)
+{
+	GKeyFile *key_file;
+	const gchar *val_str;
+	gboolean val_bool;
+
+	key_file = modem->modemsettings;
+	if (g_strcmp0(key, "CarrierMMSC") == 0) {
+		g_variant_get(value, "&s", &val_str);
+		save_string_settings(key_file, SETTINGS_GROUP, key,
+				     val_str, &modem->message_center);
+	} else if (g_strcmp0(key, "MMS_APN") == 0) {
+		g_variant_get(value, "&s", &val_str);
+		save_string_settings(key_file, SETTINGS_GROUP, key,
+				     val_str, &modem->mms_apn);
+	} else if (g_strcmp0(key, "CarrierMMSProxy") == 0) {
+		g_variant_get(value, "&s", &val_str);
+		if (!*val_str || g_strcmp0(val_str, "NULL") == 0)
+			val_str = NULL;
+		save_string_settings(key_file, SETTINGS_GROUP, key,
+				     val_str, &modem->MMS_proxy);
+	} else if (g_strcmp0(key, "DefaultModemNumber") == 0) {
+		g_variant_get(value, "&s", &val_str);
+		if (!*val_str || g_strcmp0(val_str, "NULL") == 0)
+			val_str = NULL;
+		save_string_settings(key_file, SETTINGS_GROUP, key,
+				     val_str, &modem->default_number);
+	} else if(g_strcmp0(key, "AutoProcessOnConnection") == 0) {
+		g_variant_get(value, "b", &val_bool);
+		if(val_bool == TRUE && val_bool != modem->auto_process_on_connection)
+			process_mms_process_message_queue(NULL);
+		save_boolean_settings(key_file, SETTINGS_GROUP, key,
+				      val_bool, &modem->auto_process_on_connection);
+	} else if(g_strcmp0(key, "AutoProcessSMSWAP") == 0) {
+		g_variant_get(value, "b", &val_bool);
+		if(val_bool == FALSE &&
+		   modem->autoprocess_sms_wap == TRUE &&
+		   modem->modem_available == TRUE)
+			mmsd_disconnect_from_sms_wap();
+		if(val_bool == TRUE &&
+		   modem->autoprocess_sms_wap == FALSE) {
+			mmsd_connect_to_sms_wap();
+			if(modem->modem_ready == TRUE)
+				mmsd_get_all_sms();
+		}
+		save_boolean_settings(key_file, SETTINGS_GROUP, key,
+				      val_bool, &modem->auto_process_on_connection);
+	} else {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static void
 handle_method_call(GDBusConnection		*connection,
 		   const gchar			*sender,
@@ -207,7 +294,7 @@ handle_method_call(GDBusConnection		*connection,
 		g_variant_builder_add_parsed (&settings_builder,
 					      "{'MMS_APN', <%s>}",
 				      	      modem->mms_apn);
-		if (modem->MMS_proxy) {
+		if (modem->MMS_proxy && *modem->MMS_proxy) {
 			g_variant_builder_add_parsed (&settings_builder,
 						      "{'CarrierMMSProxy', <%s>}",
 					      	      modem->MMS_proxy);
@@ -241,131 +328,119 @@ handle_method_call(GDBusConnection		*connection,
 
 		g_dbus_method_invocation_return_value (invocation, all_settings);
 	} else if(g_strcmp0(method_name, "ChangeSettings") == 0) {
+		g_autoptr(GVariant) value = NULL;
+		g_autofree char *key = NULL;
+  		gboolean close_settings = FALSE, has_item = FALSE;
 
-		GVariant *variantstatus;
-  		gchar *dict, *CarrierMMSC, *MMSAPN, *CarrierMMSProxy, *DefaultNumber;
-  		gboolean AutoProcessOnConnection, AutoProcessSMSWAP;
-  		gboolean close_settings = FALSE;
-
-
-		g_variant_get (parameters, "(sv)", &dict, &variantstatus);
-		DBG("Dict: %s", dict);
 		if (modem->modemsettings == NULL) {
 			close_settings = TRUE;
 			modem->modemsettings = mms_settings_open(IDENTIFIER, SETTINGS_STORE);
 		}
 
-		if(g_strcmp0(dict, "CarrierMMSC") == 0) {
-			g_variant_get (variantstatus, "s", &CarrierMMSC);
-			g_free(modem->message_center);
-			modem->message_center = g_strdup(CarrierMMSC);
-			DBG("Carrier MMSC set to %s", modem->message_center);
-			g_key_file_set_string(modem->modemsettings,
-					      SETTINGS_GROUP,
-				      	      "CarrierMMSC",
-				      	      modem->message_center);
 
-		} else if(g_strcmp0(dict, "MMS_APN") == 0) {
-			g_variant_get (variantstatus, "s", &MMSAPN);
-			g_free(modem->mms_apn);
-			modem->mms_apn = g_strdup(MMSAPN);
-			DBG("MMS APN set to %s", modem->mms_apn);
-			g_key_file_set_string(modem->modemsettings,
-					      SETTINGS_GROUP,
-					      "MMS_APN",
-					      modem->mms_apn);
+		g_variant_get (parameters, "(sv)", &key, &value);
+		if (dbus_change_settings (key, value))
+			has_item = TRUE;
 
-		} else if(g_strcmp0(dict, "CarrierMMSProxy") == 0) {
-			g_variant_get (variantstatus, "s", &CarrierMMSProxy);
-			g_free(modem->MMS_proxy);
-			modem->MMS_proxy = g_strdup(CarrierMMSProxy);
-
-			g_key_file_set_string(modem->modemsettings,
-					      SETTINGS_GROUP,
-				      	      "CarrierMMSProxy",
-				      	      modem->MMS_proxy);
-
-			if(g_strcmp0(modem->MMS_proxy, "NULL") == 0 || strlen (dict) < 1) {
-				g_free(modem->MMS_proxy);
-				modem->MMS_proxy = NULL;
-			}
-			DBG("MMS Proxy set to %s", modem->MMS_proxy);
-
-		} else if(g_strcmp0(dict, "DefaultModemNumber") == 0) {
-			g_variant_get (variantstatus, "s", &DefaultNumber);
-			g_free(modem->default_number);
-			modem->default_number = g_strdup(DefaultNumber);
-
-			g_key_file_set_string(modem->modemsettings,
-					      SETTINGS_GROUP,
-				      	      "DefaultModemNumber",
-				      	      modem->default_number);
-
-			if(g_strcmp0(modem->default_number, "NULL") == 0) {
-				g_free(modem->default_number);
-				modem->default_number = NULL;
-			}
-			DBG("Default Modem Number set to %s", modem->default_number);
-
-		} else if(g_strcmp0(dict, "AutoProcessOnConnection") == 0) {
-			g_variant_get (variantstatus, "b", &AutoProcessOnConnection);
-			if (modem->auto_process_on_connection == FALSE && AutoProcessOnConnection == TRUE) {
-				process_mms_process_message_queue(NULL);
-			}
-			modem->auto_process_on_connection = AutoProcessOnConnection;
-			DBG("AutoProcessOnConnection set to %d", modem->auto_process_on_connection);
-			g_key_file_set_boolean(modem->modemsettings,
-					       SETTINGS_GROUP,
-					       "AutoProcessOnConnection",
-					       modem->auto_process_on_connection);
-
-		} else if(g_strcmp0(dict, "AutoProcessSMSWAP") == 0) {
-			g_variant_get (variantstatus, "b", &AutoProcessSMSWAP);
-			if (AutoProcessSMSWAP == FALSE &&
-					modem->autoprocess_sms_wap == TRUE &&
-					modem->modem_available == TRUE) {
-				mmsd_disconnect_from_sms_wap();
-			}
-			if (AutoProcessSMSWAP == TRUE &&
-					modem->autoprocess_sms_wap == FALSE) {
-				mmsd_connect_to_sms_wap();
-				if (modem->modem_ready == TRUE) {
-					mmsd_get_all_sms();
-				}
-			}
-			modem->autoprocess_sms_wap = AutoProcessSMSWAP;
-			DBG("AutoProcessSMSWAP set to %d", modem->autoprocess_sms_wap);
-			g_key_file_set_boolean(modem->modemsettings,
-					       SETTINGS_GROUP,
-					       "AutoProcessSMSWAP",
-					       modem->autoprocess_sms_wap);
-
-		} else {
-			if (close_settings == TRUE) {
-				mms_settings_close(IDENTIFIER, SETTINGS_STORE,
-						   modem->modemsettings, FALSE);
-				modem->modemsettings = NULL;
-			}
+		if (close_settings == TRUE) {
+			mms_settings_close(IDENTIFIER, SETTINGS_STORE,
+					   modem->modemsettings, has_item);
+			modem->modemsettings = NULL;
+		} else if (has_item) {
+			mms_settings_sync(IDENTIFIER,
+					  SETTINGS_STORE,
+					  modem->modemsettings);
+		}
 
+		if (!has_item) {
 			g_dbus_method_invocation_return_error (invocation,
 							       G_DBUS_ERROR,
 							       G_DBUS_ERROR_INVALID_ARGS,
 							       "Cannot find the Property requested!");
 			return;
 		}
+
+		g_dbus_method_invocation_return_value (invocation, NULL);
+		if (!modem->MMS_proxy)
+			modem->MMS_proxy = g_strdup("NULL");
+
+		if (!*modem->MMS_proxy) {
+			g_clear_pointer (&modem->MMS_proxy, g_free);
+			modem->MMS_proxy = g_strdup("NULL");
+		}
+
+		g_dbus_connection_emit_signal(connection,
+					      NULL,
+					      MMS_PATH,
+					      MMS_MODEMMANAGER_INTERFACE,
+					      "SettingsChanged",
+					      g_variant_new("(sss)", modem->mms_apn, modem->message_center, modem->MMS_proxy),
+					      NULL);
+
+		if(g_strcmp0(modem->MMS_proxy, "NULL") == 0) {
+			g_clear_pointer (&modem->MMS_proxy, g_free);
+		}
+
+	} else if (g_strcmp0(method_name, "ChangeAllSettings") == 0) {
+		g_autoptr(GVariant) properties = NULL;
+		GVariantIter *iter;
+		GVariant *value;
+		char *key;
+  		gboolean close_settings = FALSE, has_item = FALSE;
+
+		if (modem->modemsettings == NULL) {
+			close_settings = TRUE;
+			modem->modemsettings = mms_settings_open(IDENTIFIER, SETTINGS_STORE);
+		}
+
+		g_variant_get(parameters, "(@a{sv})", &properties);
+		g_variant_get(properties, "a{sv}", &iter);
+
+		while (g_variant_iter_loop(iter, "{sv}", &key, &value)) {
+			if (dbus_change_settings(key, value))
+				has_item = TRUE;
+		}
+
 		if (close_settings == TRUE) {
 			mms_settings_close(IDENTIFIER, SETTINGS_STORE,
-					   modem->modemsettings, TRUE);
+					   modem->modemsettings, has_item);
 			modem->modemsettings = NULL;
-		} else {
+		} else if (has_item) {
 			mms_settings_sync(IDENTIFIER,
 					  SETTINGS_STORE,
 					  modem->modemsettings);
 		}
 
+		if (!has_item) {
+			g_dbus_method_invocation_return_error (invocation,
+							       G_DBUS_ERROR,
+							       G_DBUS_ERROR_INVALID_ARGS,
+							       "Cannot find the Property requested!");
+			return;
+		}
+
 		g_dbus_method_invocation_return_value (invocation, NULL);
-	}
-	else if(g_strcmp0(method_name, "ProcessMessageQueue") == 0) {
+		if (!modem->MMS_proxy)
+			modem->MMS_proxy = g_strdup("NULL");
+
+		if (!*modem->MMS_proxy) {
+			g_clear_pointer (&modem->MMS_proxy, g_free);
+			modem->MMS_proxy = g_strdup("NULL");
+		}
+
+		g_dbus_connection_emit_signal(connection,
+					      NULL,
+					      MMS_PATH,
+					      MMS_MODEMMANAGER_INTERFACE,
+					      "SettingsChanged",
+					      g_variant_new("(sss)", modem->mms_apn, modem->message_center, modem->MMS_proxy),
+					      NULL);
+
+		if(g_strcmp0(modem->MMS_proxy, "NULL") == 0) {
+			g_clear_pointer (&modem->MMS_proxy, g_free);
+		}
+
+	} else if(g_strcmp0(method_name, "ProcessMessageQueue") == 0) {
 		if(modem->modem_ready == TRUE) {
 			process_mms_process_message_queue(NULL);
 			g_dbus_method_invocation_return_value(invocation, NULL);
@@ -414,7 +489,7 @@ process_mms_process_message_queue(gpointer user_data)
 		 */
 		g_timeout_add_seconds (1, mmsd_mm_activate_bearer, NULL);
 	} else {
-		mms_error("Modem is not ready to process any unsent/unreceived MMS messages.");
+		g_critical("Modem is not ready to process any unsent/unreceived MMS messages.");
 	}
 	return FALSE;
 }
@@ -433,7 +508,6 @@ mmsd_process_sms(MMSms	*sms)
 
 	if(message) {
 		DBG("This is a regular SMS.");
-		DBG("Message: %s", message);
 	} else if(data) {
 		DBG("Received SMS WAP!");
 		if(modem->modem_ready == TRUE) {
@@ -446,13 +520,13 @@ mmsd_process_sms(MMSms	*sms)
 							  (GAsyncReadyCallback)cb_sms_delete_finish,
 							  sms);
 			} else {
-				mms_error("mmsd-tng cannot process MMS at this time!");
+				g_critical("mmsd-tng cannot process MMS at this time!");
 			}
 		} else {
-			mms_error("Modem is not connected!");
+			g_critical("Modem is not connected!");
 		}
 	} else {
-		mms_error("Not sure what kind of SMS this is!");
+		g_critical("Not sure what kind of SMS this is!");
 	}
 }
 
@@ -525,19 +599,19 @@ cb_sms_list_new_ready(MMModemMessaging	*modemmessaging,
 	list = mm_modem_messaging_list_finish(modemmessaging, result, &error);
 
 	if(error) {
-		mms_error("Couldn't get SMS list - error: %s",
+		g_critical("Couldn't get SMS list - error: %s",
 			  error->message);
 	} else {
-	for(l = list; l; l = g_list_next(l)) {
-		//We are searching for the SMS from the list that is new
-		if(!g_strcmp0(mm_sms_get_path(MM_SMS(l->data)), path)) {
-			sms = g_object_ref(MM_SMS(l->data));
-			mmsd_check_pdu_type(sms);
-			break;
+		for(l = list; l; l = g_list_next(l)) {
+			//We are searching for the SMS from the list that is new
+			if(!g_strcmp0(mm_sms_get_path(MM_SMS(l->data)), path)) {
+				sms = g_object_ref(MM_SMS(l->data));
+				mmsd_check_pdu_type(sms);
+				break;
+			}
 		}
-	}
-	g_list_free_full(list, g_object_unref);
-	g_free(path);
+		g_list_free_full(list, g_object_unref);
+		g_free(path);
 	}
 }
 
@@ -548,6 +622,7 @@ cb_dbus_signal_sms_added(MMModemMessaging	*device,
 {
 	gchar *path;
 	path = g_strdup(const_path);
+	DBG("Got new SMS");
 	mm_modem_messaging_list(modem->modem_messaging,
 				NULL,
 				(GAsyncReadyCallback)cb_sms_list_new_ready,
@@ -573,29 +648,18 @@ cb_sms_list_all_ready(MMModemMessaging	*modemmessaging,
 			sms = g_object_ref(MM_SMS(l->data));
 			mmsd_check_pdu_type(sms);
 		}
+		g_list_free_full(list, g_object_unref);
 	}
 }
 
-static gboolean
-mmsd_get_all_sms_timeout(gpointer user_data)
-{
-	DBG("Removing timeout to mmsd_get_all_sms()");
-	modem->get_all_sms_timeout = FALSE;
-
-	return FALSE;
-}
-
 static gboolean
 mmsd_mm_get_messaging_list(gpointer user_data)
 {
+	DBG("Searching for all SMS");
 	mm_modem_messaging_list(modem->modem_messaging,
 				NULL,
 				(GAsyncReadyCallback)cb_sms_list_all_ready,
 				NULL);
-	modem->get_all_sms_timeout = TRUE;
-	DBG("Adding timeout to mmsd_get_all_sms()");
-	//Adding five second timeout
-	g_timeout_add_seconds(5, mmsd_get_all_sms_timeout, NULL);
 
 	return FALSE;
 }
@@ -607,14 +671,10 @@ mmsd_get_all_sms(void)
 
 
 	if (modem->get_all_sms_timeout == FALSE) {
-		DBG("Searching for any new SMS WAPs...");
+		DBG("Queue Search for all SMS");
+		modem->get_all_sms_timeout = TRUE;
+		g_timeout_add_seconds (5, mmsd_mm_get_messaging_list, NULL);
 
-		// This is needed in case mmsd-tng is
-		// trying to come out of suspend
-		g_timeout_add_seconds (1, mmsd_mm_get_messaging_list, NULL);
-
-	} else {
-		DBG("mmsd_get_all_sms() timed out!");
 	}
 }
 
@@ -623,7 +683,7 @@ mmsd_disconnect_from_sms_wap(void)
 {
 	MmGdbusModemMessaging *gdbus_sms;
 	if (modem->modem_messaging == NULL) {
-		mms_error("SMS WAP Disconnect: There is no modem with messaging capabilities!");
+		g_critical("SMS WAP Disconnect: There is no modem with messaging capabilities!");
 		return;
 	}
 	DBG("Stopping watching SMS WAPs");
@@ -637,7 +697,7 @@ mmsd_connect_to_sms_wap(void)
 {
 	MmGdbusModemMessaging *gdbus_sms;
 	if (modem->modem_messaging == NULL) {
-		mms_error("SMS WAP Connect: There is no modem with messaging capabilities!");
+		g_critical("SMS WAP Connect: There is no modem with messaging capabilities!");
 		return;
 	}
 	gdbus_sms = MM_GDBUS_MODEM_MESSAGING(modem->modem_messaging);
@@ -723,14 +783,14 @@ mmsd_mm_add_object(MMObject *obj)
 		return;
 	}
 
-	if (modem->default_number != NULL) {
+	if (modem->default_number && *modem->default_number) {
 		g_autofree char *default_number_formatted = NULL;
 		DBG("Checking if this modem number matches default number");
 		sim = mm_modem_get_sim_sync(mm_object_get_modem(MM_OBJECT(obj)),
 						   NULL,
 				 		   &error);
 		if(error != NULL) {
-		        mms_error ("Error Getting Sim: %s", error->message);
+		        g_critical ("Error Getting Sim: %s", error->message);
 			return;
 		}
 		country_code = get_country_iso_for_mcc (mm_sim_get_imsi(sim));
@@ -748,18 +808,18 @@ mmsd_mm_add_object(MMObject *obj)
 		}
 
 		if (modem_number_ref == NULL) {
-			mms_error("Could not get number!");
+			g_critical("Could not get number!");
 			return;
 		}
 		default_number_formatted = phone_utils_format_number_e164(modem->default_number, country_code, FALSE);
 
 		if (default_number_formatted == NULL) {
-			mms_error("Default number not formatted correctly!");
+			g_critical("Default number not formatted correctly!");
 			return;
 		}
 
 		if(g_strcmp0(modem_number_formatted, default_number_formatted) != 0) {
-			mms_error("This modem does not match default number!");
+			g_critical("This modem does not match default number!");
 			return;
 		}
 	} else {
@@ -813,7 +873,7 @@ cb_object_added(GDBusObjectManager 	*manager,
 {
 	DBG("%s", __func__);
 	if(mm_object_peek_modem_messaging(MM_OBJECT(object))) {
-		mms_error("New Object does not have Messaging feature, ignoring....");
+		g_message("New Object with Messaging feature found, Adding...");
 		mmsd_mm_add_object(MM_OBJECT(object));
 	}
 
@@ -907,7 +967,7 @@ cb_mm_manager_new(GDBusConnection 	*connection,
 		mmsd_mm_get_modems();
 
 	} else {
-		mms_error("Error connecting to ModemManager: %s\n", error->message);
+		g_critical("Error connecting to ModemManager: %s\n", error->message);
 
 		mmsd_mm_state(MMSD_MM_STATE_NO_MANAGER);
 	}
@@ -916,8 +976,6 @@ cb_mm_manager_new(GDBusConnection 	*connection,
 static void
 mmsd_mm_get_modem_state(void)
 {
-	g_autoptr(GError) error = NULL;
-
 	if(!modem->modem) {
 		mmsd_mm_state(MMSD_MM_STATE_NO_MODEM);
 		return;
@@ -944,8 +1002,6 @@ mmsd_mm_get_modem_state(void)
 				return;
 			case MM_MODEM_STATE_DISABLED:
 				DBG("MM_MODEM_STATE_DISABLED");
-				DBG("Turning on Modem....");
-				mm_modem_set_power_state_sync(modem->modem, MM_MODEM_POWER_STATE_ON, NULL, &error);
 				mmsd_mm_state(MMSD_MM_STATE_MODEM_DISABLED);
 				return;
 			case MM_MODEM_STATE_DISABLING:
@@ -999,7 +1055,7 @@ mmsd_mm_state(int state)
 				mmsd_modem_unavailable();
 				DBG("Modem vanished, Disabling plugin");
 			} else {
-				mms_error("Could not connect to modem");
+				g_critical("Could not connect to modem");
 			}
 			modem->modem_available = FALSE;
 			modem->modem_ready = FALSE;
@@ -1034,7 +1090,7 @@ mmsd_mm_state(int state)
 				modem->modem_available = FALSE;
 				modem->modem_ready = FALSE;
 			} else {
-				mms_error("Could not connect to ModemManager");
+				g_critical("Could not connect to ModemManager");
 			}
 			modem->manager_available = FALSE;
 			DBG("MMSD_MM_STATE_NO_MANAGER");
@@ -1112,7 +1168,7 @@ static void bearer_handler(mms_bool_t    active,
 
 	DBG("checking for failure");
 	if(active == FALSE && modem->context_active == FALSE) {
-		mms_error("Context not active!");
+		g_critical("Context not active!");
 		mms_service_bearer_notify(modem->service, FALSE, NULL, NULL);
 		return;
 	} else {
@@ -1131,8 +1187,8 @@ find_settings_cb(const char   *apn,
 		 gpointer      user_data)
 {
 	if (error) {
-		mms_warn("Could not find settings: %s", error->message);
-		mms_warn("Your MMS settings are not in the database! Please file a merge request at https://gitlab.gnome.org/GNOME/mobile-broadband-provider-info so they can be added");
+		g_warning("Could not find settings: %s", error->message);
+		g_warning("Your MMS settings are not in the database! Please file a merge request at https://gitlab.gnome.org/GNOME/mobile-broadband-provider-info so they can be added");
 	} else {
 		DBG("Found settings: APN:%s, mmsc: %s, Proxy: %s", apn, mmsc, proxy);
 		if (mmsc != NULL) {
@@ -1154,7 +1210,7 @@ find_settings_cb(const char   *apn,
 
 			g_free(modem->MMS_proxy);
 			modem->MMS_proxy = NULL;
-			if (proxy || strlen (proxy) > 1)
+			if (proxy && *proxy)
 				modem->MMS_proxy = g_strdup(proxy);
 			else
 				modem->MMS_proxy = g_strdup("NULL");
@@ -1184,7 +1240,7 @@ find_settings_cb(const char   *apn,
 			/* Reprocess message queue since settings are right now */
 			g_timeout_add_seconds (1, process_mms_process_message_queue, NULL);
 		} else {
-			mms_warn("Your MMS settings are not in the database! Please file a merge request at https://gitlab.gnome.org/GNOME/mobile-broadband-provider-info so they can be added");
+			g_warning("Your MMS settings are not in the database! Please file a merge request at https://gitlab.gnome.org/GNOME/mobile-broadband-provider-info so they can be added");
 		}
 
 	}
@@ -1234,7 +1290,6 @@ static int set_context(void)
 					DBG("Reported State: %d", modem->state);
 					interface_disconnected = TRUE;
 				} else if(!bearer_connected) {
-					mms_error("The proper context is not connected!");
 					interface_disconnected = TRUE;
 				} else {
 					DBG("You are connected to the correct APN! Enabling context...");
@@ -1250,6 +1305,7 @@ static int set_context(void)
 		if(!modem->context_active) { // I did not find the right context I wanted.
 			if(active_bearers == max_bearers) {
 				if(interface_disconnected) {
+					g_critical("The proper context is not connected!");
 					return MMSD_MM_MODEM_INTERFACE_DISCONNECTED;
 				} else {
 					DBG("The modem is not connected to the correct APN!");
@@ -1267,17 +1323,17 @@ static int set_context(void)
 				 *	 The Pinephone and Librem 5 only support
 				 *	 one active bearer as well
 				 */
-				mms_error("This is a stub for adding a new context/bearer, but Modem Manager does not support this yet.");
+				g_critical("This is a stub for adding a new context/bearer, but Modem Manager does not support this yet.");
 				return MMSD_MM_MODEM_FUTURE_CASE_DISCONNECTED;
 			}
 		}
 	} else {
-		mms_error("There are no modem bearers! Please enable modem data");
+		g_critical("There are no modem bearers! Please enable modem data");
 		return MMSD_MM_MODEM_NO_BEARERS_ACTIVE;
 	}
 
 	if(g_strcmp0(modem->message_center, "http://mms.invalid") == 0) {
-		mms_error("The MMSC is not configured! Please configure the MMSC and restart mmsd-tng.");
+		g_critical("The MMSC is not configured! Please configure the MMSC and restart mmsd-tng.");
 		return MMSD_MM_MODEM_MMSC_MISCONFIGURED;
 	}
 	return MMSD_MM_MODEM_CONTEXT_ACTIVE;
@@ -1335,8 +1391,7 @@ static void mmsd_modem_available(void)
 							      	      g_variant_new("(sss)", modem->mms_apn, modem->message_center, modem->MMS_proxy),
 							      	      NULL);
 
-					g_free(modem->MMS_proxy);
-					modem->MMS_proxy = NULL;
+					g_clear_pointer (&modem->MMS_proxy, g_free);
 
 				}
 				g_free(modem->registered_imsi);
@@ -1350,7 +1405,7 @@ static void mmsd_modem_available(void)
 			mms_service_set_country_code(modem->service,
 						     mm_sim_get_imsi(modem->sim));
 		} else {
-		        mms_error ("Error Getting Sim: %s", error->message);
+		        g_critical ("Error Getting Sim: %s", error->message);
 		}
 
 		modem_number_ref = mm_modem_get_own_numbers (modem->modem);
@@ -1379,7 +1434,7 @@ static void mmsd_modem_available(void)
 		modem->state = mm_modem_get_state(modem->modem);
 		mmsd_mm_get_modem_state();
 	} else {
-		mms_error("Something very bad happened at mmsd_modem_available()!");
+		g_critical("Something very bad happened at mmsd_modem_available()!");
 	}
 
 }
@@ -1394,6 +1449,7 @@ static void mmsd_modem_unavailable(void)
 	if(modem->autoprocess_sms_wap) {
 		mmsd_disconnect_from_sms_wap();
 	}
+	modem->get_all_sms_timeout = FALSE;
 	g_signal_handler_disconnect(gdbus_modem,
 				    modem->modem_state_watch_id);
 	g_object_unref(modem->sim);
@@ -1421,7 +1477,7 @@ static int modemmanager_init(void)
 	modem = g_try_new0(struct modem_data, 1);
 
 	if(modem == NULL) {
-		mms_error("Could not allocate space for modem data!");
+		g_critical("Could not allocate space for modem data!");
 		return -ENOMEM;
 	}
 	modem->service = mms_service_create();
@@ -1433,7 +1489,7 @@ static int modemmanager_init(void)
 						      SETTINGS_GROUP,
 						      "CarrierMMSC", &error);
 	if(error) {
-		mms_error("No MMSC was configured!");
+		g_critical("No MMSC was configured!");
 		modem->message_center = g_strdup("http://mms.invalid");
 		g_key_file_set_string(modem->modemsettings, SETTINGS_GROUP,
 				      "CarrierMMSC", modem->message_center);
@@ -1444,7 +1500,7 @@ static int modemmanager_init(void)
 					       SETTINGS_GROUP,
 					       "MMS_APN", &error);
 	if(error) {
-		mms_error("No MMS APN was configured!");
+		g_critical("No MMS APN was configured!");
 		modem->mms_apn = g_strdup("apn.invalid");
 		g_key_file_set_string(modem->modemsettings, SETTINGS_GROUP,
 							  "MMS_APN", modem->mms_apn);
@@ -1455,31 +1511,29 @@ static int modemmanager_init(void)
 						 SETTINGS_GROUP,
 						 "CarrierMMSProxy", &error);
 	if(error) {
-		mms_error("Setting MMS Procy to NULL");
+		g_critical("Setting MMS Procy to NULL");
 		modem->MMS_proxy = g_strdup("NULL");
 		g_key_file_set_string(modem->modemsettings, SETTINGS_GROUP,
 				      "CarrierMMSProxy", modem->MMS_proxy);
 		g_clear_error(&error);
 	}
-	if(g_strcmp0(modem->MMS_proxy, "NULL") == 0 || strlen (modem->MMS_proxy) < 1) {
-		g_free(modem->MMS_proxy);
-		modem->MMS_proxy = NULL;
+	if(g_strcmp0(modem->MMS_proxy, "NULL") == 0 || !*modem->MMS_proxy) {
+		g_clear_pointer (&modem->MMS_proxy, g_free);
 	}
 
 	modem->default_number = g_key_file_get_string(modem->modemsettings,
 						 SETTINGS_GROUP,
 						 "DefaultModemNumber", &error);
 	if(error) {
-		mms_error("No Default Modem Number was configured! Setting to NULL.");
+		g_critical("No Default Modem Number was configured! Setting to NULL.");
 		modem->default_number = g_strdup("NULL");
 
 		g_key_file_set_string(modem->modemsettings, SETTINGS_GROUP,
 				      "DefaultModemNumber", modem->default_number);
 		g_clear_error(&error);
 	}
-	if(g_strcmp0(modem->default_number, "NULL") == 0) {
-		g_free(modem->default_number);
-		modem->default_number = NULL;
+	if(g_strcmp0(modem->default_number, "NULL") == 0 || !*modem->default_number) {
+		g_clear_pointer (&modem->default_number, g_free);
 	}
 
 	modem->auto_process_on_connection = g_key_file_get_boolean(modem->modemsettings,
@@ -1487,7 +1541,7 @@ static int modemmanager_init(void)
 								   "AutoProcessOnConnection",
 								   &error);
 	if(error) {
-		mms_error("Auto Process On Connection was not configured! Setting to TRUE.");
+		g_critical("Auto Process On Connection was not configured! Setting to TRUE.");
 		modem->auto_process_on_connection = TRUE;
 		g_key_file_set_boolean(modem->modemsettings, SETTINGS_GROUP,
 				       "AutoProcessOnConnection", modem->auto_process_on_connection);
@@ -1499,7 +1553,7 @@ static int modemmanager_init(void)
 							   "AutoProcessSMSWAP",
 							   &error);
 	if(error) {
-		mms_error("Auto Process SMS WAP was not configured! Setting to TRUE.");
+		g_critical("Auto Process SMS WAP was not configured! Setting to TRUE.");
 		modem->autoprocess_sms_wap = TRUE;
 		g_key_file_set_boolean(modem->modemsettings, SETTINGS_GROUP,
 				       "AutoProcessSMSWAP", modem->autoprocess_sms_wap);
@@ -1536,7 +1590,7 @@ static int modemmanager_init(void)
 								   NULL,	/* user_data_free_func */
 								   &error);	/* GError** */
 	if(error) {
-		mms_error("Error registering MMSD ModemManager interface: %s\n", error->message);
+		g_critical("Error registering MMSD ModemManager interface: %s\n", error->message);
 		g_clear_error(&error);
 	}
 
@@ -1556,6 +1610,7 @@ static void modemmanager_exit(void)
 	GDBusConnection	*connection = mms_dbus_get_connection ();
 	if(modem->plugin_registered == TRUE) {
 		mms_service_unregister(modem->service);
+		mms_service_unref(modem->service);
 		modem->modemsettings = NULL;
 	}
 	if(modem->modem_available) {
