@@ -831,10 +831,10 @@ static gboolean mms_parse_headers(struct wsp_header_iter *iter,
 		handler = handler_for_type(h);
 		if (handler == NULL) {
 		    if(h == MMS_HEADER_INVALID) {
-		      mms_error("Got MMS_HEADER_INVALID: 0x%02X. Returning False", h);
+		      g_critical("Got MMS_HEADER_INVALID: 0x%02X. Returning False", h);
 			  return FALSE;
 		    } else if (h == __MMS_HEADER_MAX) {
-		      mms_error("Got __MMS_HEADER_MAX: 0x%02X. Returning False", h);
+		      g_critical("Got __MMS_HEADER_MAX: 0x%02X. Returning False", h);
 			  return FALSE;
 		    } else {
 		      DBG("Header 0x%02X is not handled in decoding. Skipping....", h);
